const express = require('express')
const mongoose = require('mongoose')
const dotenv = require('dotenv')
const bodyparser = require('body-parser');
const cors = require('cors');
const line = require('@line/bot-sdk');
const { stallStatusModel, toiletModel } = require('./schema')

dotenv.config()
mongoose.connect( process.env.MONGODBURI, { useNewUrlParser : true , useUnifiedTopology : true, useFindAndModify : true } )

const app = express()
const db = mongoose.connection


app.use( bodyparser.json() )
app.use( cors() )

db.on('error', console.error.bind( console, 'mongodb connection error' ))

app.get( '/', ( req, res ) => {
  res.send( 'hello world' );
} )

app.post( '/register/', async ( req, res ) => {
  try{
    const stallId = req.body.stallId
    const toiletId = req.body.toiletId

    if ( !stallId ){
      const stallRecord = new stallStatusModel({ isVacant : false })
      const record = await stallRecord.save()
      await toiletModel.findByIdAndUpdate( toiletId, { '$push' : { 'stallId' : record._id } } )
      res.status(200).send(String(record._id))
    } else { 
      const record = await stallStatusModel.findById( stallId )
      if ( !record ){
        res.sendStatus(403)
      }
    }
  } catch {
    res.sendStatus(400)
  }
})

app.post( '/status/:stallId', async ( req, res ) => {
  try{
    const stallId = req.params.stallId

    await stallStatusModel.findByIdAndUpdate( 
      stallId,
      { '$set' : { 
        'isVacant' : req.body.status
      } }
    )

    res.sendStatus(200)
  } catch {
    res.sendStatus(400)
  }
} )

app.get( '/status/:toiletId', async ( req, res ) => { 
  try { 
    const toiletId = req.params.toiletId
    let records = await toiletModel.findById( toiletId )
    records = await stallStatusModel.find( { '_id' : { '$in' : records.stallId.map( (elm) => mongoose.Types.ObjectId(elm) ) } } ) 

    let count = 0

    for ( const record of records ){
      if ( record.isVacant ){
        count += 1
      }
    }

    res.status(200).send( count + '/' + records.length )

  } catch( err ) { 
    res.sendStatus(500)
  }
})

app.get( '/status/', async ( req, res ) => {
  try {
    const toilets = await toiletModel.find().populate( 'stallId' );
    return res.status(200).send( 
      toilets.map( ( record ) => {
        return {
        'location' : record.location,
        'name' : record.name,
        'vacant' : record.stallId.map( ( status ) => status.isVacant  ),
      }} ))

  } catch ( err ) {
    res.sendStatus(500)
  }
})

app.listen( 3000, () => { 
  console.log('backend is running!')
})
