const express = require('express')
const line = require('@line/bot-sdk');
const mongoose = require('mongoose')
const dotenv = require('dotenv')
const { stallStatusModel, toiletModel } = require('./schema')

dotenv.config()
mongoose.connect( process.env.MONGODBURI, { useNewUrlParser : true , useUnifiedTopology : true, useFindAndModify : true } )

const config = {
  channelAccessToken : process.env.CHANNELACCESSTOKEN,
  channelSecret : process.env.CHANNELSECRET
}
const app = express()
const db = mongoose.connection
const client = new line.Client(config)

db.on('error', console.error.bind( console, 'mongodb connection error' ))

app.post( '/emergency/:stallId', async ( req, res ) => { 
  try { 
    const groupId = 'C8c4b8bc31bedcbde3bd3f79a9627fedd'
    const stallId = req.params.stallId
    const toilet = await toiletModel.findOne( { 'stallId' : { '$in' : [ stallId ] } } )
    const text = 'Someone has stay in ' + toilet.location + ' for too long'

    client.pushMessage( groupId, { type : 'text', text : text } )
    res.sendStatus(200)
  } catch ( err ) {
    res.sendStatus(500)
  }
} )

app.post( '/webhook', line.middleware( config ), async ( req, res ) => { 
  const handleEvent = ( event ) => { 
    if ( event.type !== 'message' || event.message.type !== 'text' ){
      return Promise.resolve(null)
    }

    if ( event.message.text == 'safe' ){
      return client.replyMessage( event.replyToken, { type : 'text', text : 'Recieve Confirmation' } )
    } else if ( event.message.text == 'unsafe' ) { 
      return client.replyMessage( event.replyToken, { type : 'text', text : 'Please proceed emergency protcol' } )
    }
  }

  Promise
    .all( req.body.events.map( handleEvent ) )
    .then( ( result ) => res.json( result ) )
})


app.listen( 3001, () => { 
  console.log('backend is running!')
})
